@@ -1,13 +1,14 @@
 ;;; x-face.el -- X-Face encoder/decoder/viewer for Emacsen.
 
-;; Copyright (C) 1996-2002 Katsumi Yamaoka
-;; Copyright (C) 1996-2002 Tatsuya Ichikawa
-;; Copyright (C) 1996-2002 OKUNISHI Fujikazu
-;; Copyright (C) 1996-2002 KOSEKI Yoshinori
-;; Copyright (C) 1996-2002 Ken'ichi OKADA
-;; Copyright (C) 1996-2002 Tetsuya Uemura
-;; Copyright (C) 1996-2002 TAKAHASHI Kaoru
-;; Copyright (C) 2002      Daiki Ueno
+;; Copyright (C) 1996-2003 Katsumi Yamaoka
+;; Copyright (C) 1996-2003 Tatsuya Ichikawa
+;; Copyright (C) 1996-2003 OKUNISHI Fujikazu
+;; Copyright (C) 1996-2003 KOSEKI Yoshinori
+;; Copyright (C) 1996-2003 Ken'ichi OKADA
+;; Copyright (C) 1996-2003 Tetsuya Uemura
+;; Copyright (C) 1996-2003 TAKAHASHI Kaoru
+;; Copyright (C) 2002-2003 Daiki Ueno
+;; Copyright (C) 2003      Tatsuya Kinoshita
 ;; Author: Katsumi Yamaoka   <yamaoka@jpl.org>
 ;;         Tatsuya Ichikawa  <ichikawa@jpl.org>
 ;;         OKUNISHI Fujikazu <fuji0924@mbox.kyoto-inet.or.jp>
@@ -16,9 +17,10 @@
 ;;         Tetsuya Uemura    <uemura@pp.iij4u.or.jp>
 ;;         TAKAHASHI Kaoru   <kaoru@kaisei.org>
 ;;         Daiki Ueno        <ueno@unixuser.org>
+;;         Tatsuya Kinoshita <tats@vega.ocn.ne.jp>
 ;; Maintainer: Katsumi Yamaoka <yamaoka@jpl.org>
 ;; Created: 1996/09/19
-;; Revised: 2002/02/25
+;; Revised: 2003/08/26
 ;; Keywords: x-face, x-face-mule, semi-gnus, gnus, mew, mail, mh-e,
 ;;           rmail, vm, cmail, wl
 
@@ -37,21 +39,42 @@
 ;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 ;; Boston, MA 02111-1307, USA.
 
-;; Commands:
-;;  x-face-encode
-;;	"Generate X-Face string(s) from xbm file."
-;;  x-face-insert
-;;	"Insert X-Face fields."
-;;  x-face-save
-;;	"Save X-Face fields to files."
-;;  x-face-view
-;;	"View X-Face fields."
-;;  x-face-ascii-view
-;;	"View X-Face fields as ascii pictures."
+;;; Commentary:
+
+;; Now the X-Face utility for Emacs 21 is available at:
+;;
+;; ftp://ftp.jpl.org/pub/elisp/x-face-e21.el.gz
+;;
+;; We strongly recommend you use it rather than this module if you are
+;; using Emacs 21 having a new redisplay engine.
+
+;; This module provides the following five commands.  Each command name
+;; expresses its function literally:
+;;
+;; x-face-encode
+;;  Read an XBM file, encode it to the X-Face data, and show it.
+;; x-face-insert
+;;  Read an XBM file, encode it to the X-Face data, and insert it to
+;;  the current buffer as a message header.
+;; x-face-save
+;;  Extract X-Face headers in the current buffer, encode them to XBM
+;;  data, and save them in the local system.
+;; x-face-view
+;;  Extract X-Face headers in the current buffer, encode them to XBM
+;;  data, and show them using an external image viewer.
+;; x-face-ascii-view
+;;  Extract X-Face headers in the current buffer, encode them to XBM
+;;  data, and show them as ascii pictures in an Emacs frame."
+;;
+;; Please note, this module provides no image viewer in an Emacs frame
+;; for the historical reason.  However, it is easy to display X-Face
+;; images inline; Gnus provides the X-Face viewer by itself, you can
+;; use x-face-mule.el for Mule 2 and Emacs, x-face-e21.el for Emacs 21,
+;; x-face-xmas.el for XEmacs, and so forth.
 
 ;;; Code:
 
-(defconst x-face-version-number "1.3.6.19"
+(defconst x-face-version-number "1.3.6.24"
   "Version number for this version of x-face.")
 (defconst x-face-version
   (format "X-Face utility v%s" x-face-version-number)
@@ -94,8 +117,8 @@
 ;;  "Yellow Submarine (remix)"
 ;;  "Hey Bulldog (remix)"
 ;;  "Eleanor Rigby (remix)"
-  "Love You To (remix)"
-;;  "All Together Now (remix)"
+;;  "Love You To (remix)"
+  "All Together Now (remix)"
 ;;  "Lucy in the Sky With Diamonds (remix)"
 ;;  "Think for Yourself (remix)"
 ;;  "Sgt. Pepper's Lonely Hearts Club Band (remix)"
@@ -198,23 +221,23 @@
   (or (and (boundp 'x-face-xbm-file-directory)
 	   x-face-xbm-file-directory)
       "~/x-faces")
-  "Name of the directory where the image files are existed."
+  "Directory name where image files will be found."
   :group 'x-face
   :type 'directory)
 
 (defcustom x-face-image-file-directory-for-save
   x-face-image-file-directory
-  "Name of the directory where the image files will be saved in."
+  "Directory name where image files will be saved in."
   :group 'x-face
   :type 'directory)
 
 (defcustom x-face-default-xbm-file nil
-  "Default xbm file name of user's face."
+  "Default xbm file name for user's face."
   :group 'x-face
   :type 'string)
 
 (defcustom x-face-insert-interactive t
-  "If non-nil, x-face-insert will be called interactive."
+  "If non-nil, x-face-insert will be called interactively."
   :group 'x-face
   :type 'boolean)
 
@@ -222,7 +245,7 @@
   (and (or (not (boundp 'x-face-use-gzip-when-save))
 	   x-face-use-gzip-when-save)
        'gzip)
-  "Compress method for saving files.
+  "Compress method for saving image files.
 This variable can be set to `nil' `gzip' or 'bzip2'."
   :group 'x-face
   :type '(radio (variable-item gzip)
@@ -244,23 +267,24 @@ This variable can be set to `nil' `gzip' or 'bzip2'."
 		   "GUNZIPping")
 	   '(".gz" "gzip" "-c9" "GZIPping" "gzip" "-cd" "GUNZIPping")))
    '(bzip2 ".bz2" "bzip2" "-f" "BZIPping" "bzip2" "-d" "BUNZIPping"))
-  "List of the filename suffix, compress program, compress option,
-compress message, uncompress program, uncompress option, uncompress message."
+  "List of compression methods.  Each element looks like (METHOD
+FILENAME-SUFFIX COMPRESS-PROGRAM COMPRESS-ARGS COMPRESS-MSG
+UNCOMPRESS-PROGRAM UNCOMPRESS-ARGS UNCOMPRESS-MSG)."
   :group 'x-face
   :type 'sexp)
 
 (defcustom x-face-user-x-face-variable-name "x-face-encoded-string"
-  "Variable name of the encoded string."
+  "Variable name for the encoded X-Face string."
   :group 'x-face
   :type 'string)
 
 (defcustom x-face-compface-program "compface"
-  "Program to encode X-Face."
+  "Program used to encode X-Face."
   :group 'x-face
   :type 'string)
 
 (defcustom x-face-uncompface-program "uncompface"
-  "Program to decode X-Face."
+  "Program used to decode X-Face."
   :group 'x-face
   :type 'string)
 
@@ -268,7 +292,7 @@ compress message, uncompress program, uncompress option, uncompress message."
   (cond ((eq 'OS/2 system-type)		"pmview")
 	((eq 'windows-nt system-type)	"start")
 	(t				"xv"))
-  "Program to view image."
+  "Program used to view images."
   :group 'x-face
   :type 'string)
 
@@ -276,7 +300,7 @@ compress message, uncompress program, uncompress option, uncompress message."
   (cond ((eq 'OS/2 system-type)		nil)
 	((eq 'windows-nt system-type)	nil)
 	(t				'("-nodecor" "-")))
-  "Options for viewer program."
+  "Options used for a viewer program."
   :group 'x-face
   :type 'sexp)
 
@@ -284,7 +308,7 @@ compress message, uncompress program, uncompress option, uncompress message."
   (cond ((memq system-type '(OS/2 emx)) 'x-face-icons-to-os2-bmp)
 	((eq 'windows-nt system-type)  'x-face-icons-to-win-bmp)
 	(t 'x-face-icons-to-xpm))
-  "Function to make a color picture."
+  "Function used to make color pictures."
   :group 'x-face
   :type '(radio (function-item x-face-icons-to-os2-bmp)
 		(function-item x-face-icons-to-win-bmp)
@@ -292,7 +316,7 @@ compress message, uncompress program, uncompress option, uncompress message."
 		function))
 
 (defcustom x-face-view-asynchronous (memq system-type '(OS/2 emx windows-nt))
-  "Non-nil enables viewer asynchronous."
+  "Non-nil makes the external viewer run asynchronously."
   :group 'x-face
   :type 'boolean)
 
@@ -303,7 +327,7 @@ compress message, uncompress program, uncompress option, uncompress message."
 	 'x-face-view-asynchronous-function-for-win95)
 	(t
 	 'x-face-view-asynchronous-function-for-some))
-  "Definitions of asynchronous view function."
+  "Function used to show images asynchronously."
   :group 'x-face
   :type '(radio (function-item x-face-view-asynchronous-function-for-OS/2)
 		(function-item x-face-view-asynchronous-function-for-win95)
@@ -314,7 +338,7 @@ compress message, uncompress program, uncompress option, uncompress message."
   (cond ((eq 'OS/2 system-type)		2)
 	((eq 'emx system-type)		2)
 	(t				10))
-  "Number of seconds to wait for the starting up of the external viewer."
+  "Number of seconds to wait for the starting up the external viewer."
   :group 'x-face
   :type 'integer)
 
@@ -348,24 +372,25 @@ compress message, uncompress program, uncompress option, uncompress message."
     (wl-summary-mode
      . (x-face-wl-get-article
 	x-face-extracted-headers-buffer)))
-  "*Alist for saving or viewing. Each element should be specified as follows.
+  "Alist for saving or viewing X-Face images.  Each element should be
+specified as follows:
 
 '(MAJOR-MODE . (PREPARE-FUNCTION BUFFERorFUNCTION POST-FUNCTION))"
   :group 'x-face
   :type 'sexp)
 
 (defcustom x-face-load-hook nil
-  "Hook to be run after the x-face package has been loaded."
+  "Hook run after the x-face package is loaded."
   :group 'x-face
   :type 'hook)
 
 (defcustom x-face-insert-hook nil
-  "Hook run after inserting X-Face fields."
+  "Hook run after inserting X-Face headers."
   :group 'x-face
   :type 'hook)
 
 (defcustom x-face-add-x-face-version-header nil
-  "Non-nil forces 'x-face-insert to add X-Face-Version field. :-p"
+  "If non-nil, the X-Face-Version header will be added."
   :group 'x-face
   :type 'boolean)
 
@@ -382,13 +407,25 @@ compress message, uncompress program, uncompress option, uncompress message."
 	       (eq 'dark hilit-background-mode)))
       ["$" "\"" "0" " "]
     [" " "o" "\"" "$"])
-  "Bitmap to ascii picture conversion table.
+  "Bitmap table used to ascii picture conversion.
 \[0 0 1 1]
 \[0 1 0 1]"
   :group 'x-face
   :type '(radio (choice-item :tag "Dark" ["$" "\"" "0" " "])
 		(choice-item :tag "Light" [" " "o" "\"" "$"])))
 
+(defcustom x-face-create-directory-function
+  (if (fboundp 'make-directory)
+      'make-directory
+    (function
+     (lambda (dir)
+       (call-process "/bin/mkdir" nil nil nil dir)
+       (or (file-directory-p dir)
+	   (error "!! Can't create directory %s." dir)))))
+  "Function used to make a directory."
+  :group 'x-face
+  :type 'function)
+
 (defcustom x-face-tmp-directory
   (file-name-as-directory
    (cond ((fboundp 'temp-directory)
@@ -401,28 +438,17 @@ compress message, uncompress program, uncompress option, uncompress message."
 	  (or (getenv "TMPDIR") (getenv "TMP") (getenv "TEMP") "SYS$SCRATCH:"))
 	 (t
 	  (or (getenv "TMPDIR") (getenv "TMP") (getenv "TEMP") "/tmp"))))
-  "Name of TMP directory."
+  "Name of the temporary directory."
   :group 'x-face
   :type 'directory)
 
-(defcustom x-face-create-directory-function
-  (if (fboundp 'make-directory)
-      'make-directory
-    (function
-     (lambda (dir)
-       (call-process "/bin/mkdir" nil nil nil dir)
-       (or (file-directory-p dir)
-	   (error "!! Can't create directory %s." dir)))))
-  "Definitions of make-directory function."
-  :group 'x-face
-  :type 'function)
-
-(defcustom x-face-tmp-file-name-format
+(defcustom x-face-tmp-directory-name-format
   (let ((r 0))
     (while (<= r 0) (setq r (random)))
-    (expand-file-name (format ".x-face-%s-%d-%%d.%%s" (user-login-name) r)
-		      x-face-tmp-directory))
-  "Format for the temporary file name."
+    (format ".x-face-%s-%d-%%d" (user-login-name) r))
+  "Format used for temporary subdirectory names.
+Subdirectories will be created and deleted in the temporary directory
+specified by the `x-face-tmp-directory' variable."
   :group 'x-face
   :type 'string)
 
@@ -443,7 +469,7 @@ compress message, uncompress program, uncompress option, uncompress message."
 
 (defconst x-face-mirror-table-binary
   [0 8 4 12 2 10 6 14 1 9 5 13 3 11 7 15]
-  "Mirror conversion table for binary.")
+  "Mirror conversion table in binary.")
 
 (defconst x-face-mirror-table-8bit-binary
   (eval-when-compile
@@ -463,7 +489,7 @@ compress message, uncompress program, uncompress option, uncompress message."
 	(aset table i k)
 	(setq i (1+ i)))
       table))
-  "Mirror conversion table for 8-bit binary.")
+  "Mirror conversion table in 8-bit binary.")
 
 (defvar x-face-extracted-headers-buffer nil)
 
@@ -492,7 +518,7 @@ compress message, uncompress program, uncompress option, uncompress message."
 	    x-face-xmas-xface-p)))
 
 
-;;; Emulations for each Emacs variants.
+;;; Emulations for Emacs variants.
 (defconst x-face-no-conversion
   (cond (;; XEmacs
 	 (featurep 'xemacs) (coding-system-name (get-coding-system nil)))
@@ -548,8 +574,7 @@ compress message, uncompress program, uncompress option, uncompress message."
 	"Modify ALIST to set VALUE to ITEM.
 If there is a pair whose car is ITEM, replace its cdr by VALUE.
 If there is not such pair, create new pair (ITEM . VALUE) and
-return new alist whose car is the new pair and cdr is ALIST.
-\[tomo's ELIS like function]"
+return new alist whose car is the new pair and cdr is ALIST."
 	(let ((pair (assoc item alist)))
 	  (if pair
 	      (progn
@@ -562,14 +587,14 @@ return new alist whose car is the new pair and cdr is ALIST.
       (call-process "/bin/rm" nil nil nil "-f" filename)))
 
 
-;;; Macros and defsubsts.
+;;; Macros and inline functions.
 (defmacro x-face-following-char ()
   (if (fboundp 'char-after)
       '(char-after (point))
     '(following-char)))
 
 (defmacro x-face-read-file-name (prompt dir default mustmatch initial)
-  "Read file name."
+  "Read file name interactively."
   (let ((major (string-to-int emacs-version))
 	(minor (and (boundp 'emacs-minor-version) emacs-minor-version)))
     (cond ((> 19 major)
@@ -776,7 +801,7 @@ return new alist whose car is the new pair and cdr is ALIST.
 	  (t (apply 'error args)))))
 
 (defun x-face-expand-file-name (filename)
-  "Convert FILENAME to absolute."
+  "Convert FILENAME to absolute pathname."
   (and (stringp filename)
        (let ((name (file-name-nondirectory filename))
 	     (directory (expand-file-name (file-name-directory filename)))
@@ -831,7 +856,7 @@ return new alist whose car is the new pair and cdr is ALIST.
 	;; jam-zcat, jka-compr
 	jam-zcat-filename-list
 	jka-compr-compression-info-list)
-    (write-region (point-min) (point-max) filename nil (and silent 'silent))))
+    (write-region (point-min) (point-max) filename nil silent)))
 
 (defun x-face-call-process-buffer-no-conversion (program &rest program-args)
   (let (;; NEmacs
@@ -848,9 +873,8 @@ return new alist whose car is the new pair and cdr is ALIST.
 	;; XEmacs (>= 20.2b1)
 	(coding-system-for-read x-face-no-conversion)
 	(coding-system-for-write x-face-no-conversion))
-    (apply' call-process-region
-	    (point-min) (point-max) program t (current-buffer) nil
-	    program-args)))
+    (apply 'call-process-region (point-min) (point-max)
+	   program t (current-buffer) nil program-args)))
 
 (defun x-face-compress-buffer ()
   (let ((elt (assq x-face-compressor x-face-compressor-program-list))
@@ -936,7 +960,7 @@ return new alist whose car is the new pair and cdr is ALIST.
       ""))
 
 (defun x-face-extract-mail-address (str)
-  "Extract mail address from string."
+  "Extract mail address from a string STR."
   (cond ((string-match "^[\n\t ,]*(.*)[\n\t ,]*" str) "nobody")
 	((string-match "<\\([^\n\t ,]+\\)>" str)
 	 (substring str (match-beginning 1) (match-end 1)))
@@ -945,7 +969,7 @@ return new alist whose car is the new pair and cdr is ALIST.
 	(t "nobody")))
 
 (defun x-face-mail-fetch-mail-address ()
-  "Return the mail address from the header."
+  "Extract a mail address from the header."
   (let (end str buffer)
     (save-excursion
       (goto-char (point-min))
@@ -1242,7 +1266,7 @@ return new alist whose car is the new pair and cdr is ALIST.
    (buffer-substring-no-properties beg end)))
 
 (defun x-face-extract-x-face-fields-to-icons (&optional single glyph)
-  "Extract X-Face fields to icons. It returns the list of
+  "Extract X-Face fields to icons.  It returns the list of
 \(FIELD_or_nil TYPE X Y DATA DATA ...) or (FIELD_or_nil GLYPH)."
   (let ((case-fold-search t)
 	buffer-undo-list
@@ -1463,7 +1487,7 @@ return new alist whose car is the new pair and cdr is ALIST.
      (x-face-extract-x-face-fields-to-icons 'single))))
 
 (defun x-face-concat-mono-icons (x y icons)
-  "Concatenate multiple MONO icons."
+  "Concatenate multiple monochrome icons."
   (if (eq 1 x)
       (apply 'concat icons)
     (let (i j k m)
@@ -1535,7 +1559,7 @@ return new alist whose car is the new pair and cdr is ALIST.
     (nreverse bin)))
 
 (defun x-face-icons-to-xpm (name x y icons &optional spec)
-  "Generate XPM color picture from three icons."
+  "Generate an XPM color picture from three icons."
   (if spec
       "xpm"
     (save-excursion
@@ -1620,7 +1644,7 @@ return new alist whose car is the new pair and cdr is ALIST.
       buffer)))
 
 (defun x-face-icons-to-os2-bmp (name x y icons &optional spec)
-  "Generate OS/2 BMP color picture from three icons."
+  "Generate an OS/2 BMP color picture from three icons."
   (if spec
       "bmp"
     (let ((size (+ 74 (* 1152 x y)))
@@ -1654,7 +1678,7 @@ return new alist whose car is the new pair and cdr is ALIST.
 	)))))
 
 (defun x-face-icons-to-win-bmp (name x y icons &optional spec)
-  "Generate Windoze BMP color picture from three icons."
+  "Generate a Windoze BMP color picture from three icons."
   (if spec
       "bmp"
     (let ((size (+ 88 (* 1152 x y)))
@@ -1730,7 +1754,7 @@ return new alist whose car is the new pair and cdr is ALIST.
 
 (defun x-face-xbm-to-pbm (xbm &optional raw mirrored)
   "Return a PBM image converted from an XBM image.  If the optional RAW
-is non-nil, a raw PBM format will be used.  If the optional MIRRORED
+is non-nil, the raw PBM format will be used.  If the optional MIRRORED
 is non-nil, the return value will be right and left mirrored for each
 byte."
   (let (width height bits i rest)
@@ -1839,7 +1863,7 @@ static char * Gray_X_Face[] = {
       (buffer-string))))
 
 (defun x-face-extract-x-face-fields-to-pictures (buffer name)
-  "Extract X-Face fields to pictures."
+  "Extract X-Face fields and return pictures."
   (save-excursion
     (set-buffer buffer)
     (let ((data (x-face-extract-x-face-fields-to-icons))
@@ -1884,14 +1908,33 @@ static char * Gray_X_Face[] = {
 (defun x-face-view-buffer (spec_buffer)
   (let ((spec (car spec_buffer))
 	(buffer (cdr spec_buffer))
-	tmp-fname)
+	tmp-dir umask tmp-fname)
     (save-excursion
       (set-buffer buffer)
       (if (and x-face-view-asynchronous x-face-view-asynchronous-function)
 	  (progn
-	    (setq tmp-fname (format x-face-tmp-file-name-format
-				    x-face-tmp-file-number spec))
-	    (incf x-face-tmp-file-number)
+	    (while (file-exists-p
+		    (setq tmp-dir (expand-file-name
+				   (format x-face-tmp-directory-name-format
+					   x-face-tmp-file-number)
+				   x-face-tmp-directory)))
+	      (incf x-face-tmp-file-number))
+	    (if (fboundp 'set-default-file-modes)
+		(progn
+		  (setq umask (default-file-modes))
+		  (unwind-protect
+		      (progn
+			(set-default-file-modes 448) ;; decimal for octal 0700
+			;; Create temporary directory.  If it has already been
+			;; created by another user, an error will occur.
+			(funcall x-face-create-directory-function tmp-dir))
+		    (set-default-file-modes umask)))
+	      (funcall x-face-create-directory-function tmp-dir)
+	      (set-file-modes tmp-dir 448))
+	    ;; tmp-dir has been created
+	    (setq tmp-fname (expand-file-name (format "X-Face.%s" spec)
+					      tmp-dir))
+	    ;; tmp-fname is safe
 	    (x-face-save-buffer-no-conversion tmp-fname 'silent)
 	    (funcall x-face-view-asynchronous-function tmp-fname))
 	(apply 'x-face-call-process-buffer-no-conversion
@@ -1899,7 +1942,7 @@ static char * Gray_X_Face[] = {
     (kill-buffer buffer)))
 
 (defun x-face-save-files (buffer name)
-  "Save X-Face fields to files."
+  "Save X-Face headers to files."
   (let ((directory (x-face-file-name-as-directory
 		    x-face-image-file-directory-for-save 'noslash))
 	(re (format "^%s\\(-[0-9]+\\)?\\.\\(bmp\\|xbm\\|xpm\\)\\(%s\\)?$"
@@ -1960,7 +2003,7 @@ static char * Gray_X_Face[] = {
 	(kill-buffer buffer)))))
 
 (defun x-face-icon-to-ascii-buffer (&optional keep-trailing-white-space)
-  "Convert icon to ascii picture."
+  "Convert icon data to an ascii picture."
   (goto-char (point-min))
   (while (re-search-forward "/\\*\\(.\\|\n\\)*\\*/[\n\t ]*" nil t)
     (replace-match ""))
@@ -2020,8 +2063,12 @@ static char * Gray_X_Face[] = {
 	   (re-search-forward "^\\( *\n\\)+" nil t)
 	   (point))
        (point-min)))
-    (message "Press any key to continue.")
-    (let ((inhibit-quit t)) (read-char-exclusive))
+    (message "Press any key to continue")
+    (let ((fn (if (featurep 'xemacs)
+		  'next-command-event
+		'read-event)))
+      (funcall fn)
+      (discard-input))
     (erase-buffer)
     (insert-char ?\n 12)
     (insert "                 Please wait...")
@@ -2030,7 +2077,7 @@ static char * Gray_X_Face[] = {
     (set-window-configuration wconf)))
 
 (defun x-face-save-or-view (task)
-  "Save or view X-Face fields."
+  "Save or view X-Face headers."
   (let ((method (assq major-mode x-face-save-or-view-method-alist))
 	(msg (cond ((eq 'save task) "Sav")
 		   ((memq task '(view ascii)) "View")))
@@ -2109,7 +2156,8 @@ static char * Gray_X_Face[] = {
 
 ;;;###autoload
 (defun x-face-encode (file &optional arg)
-  "Generate X-Face string(s) from xbm file."
+  "Read an XBM file FILE, encode it to the X-Face data, and show it.
+If ARG is given, show the Emacs-Lisp form instead."
   (interactive (list (x-face-read-existing-file-name "[Encode] ")
 		     current-prefix-arg))
   (setq file (expand-file-name file))
@@ -2208,7 +2256,7 @@ static char * Gray_X_Face[] = {
 
 ;;;###autoload
 (defun x-face-menu-encode ()
-  "Generate X-Face string(s) from xbm file with menu."
+  "Generate X-Face string(s) from xbm file using menu."
   (interactive)
   (and (> (string-to-int emacs-version) 18)
        (let ((minibuffer-setup-hook minibuffer-setup-hook)
@@ -2221,7 +2269,12 @@ static char * Gray_X_Face[] = {
 
 ;;;###autoload
 (defun x-face-insert (&optional file append)
-  "Insert X-Face fields."
+  "Read an XBM file FILE, encode it to the X-Face data, and insert it to
+the current buffer as a message header.  If FILE is omitted, use a
+file specified by the `x-face-default-xbm-file' variable.  Although it
+normally deletes existing X-Face headers before inserting, you can
+insert multiple X-Face headers using the optional argument APPEND (it
+can be specified by the `C-u' key for the interactive use)."
   (interactive (list (x-face-read-existing-file-name "[Insert] ")
 		     current-prefix-arg))
   (or file
@@ -2331,7 +2384,7 @@ static char * Gray_X_Face[] = {
 
 ;;;###autoload
 (defun x-face-menu-insert ()
-  "Insert X-Face with menu."
+  "Insert an X-Face using menu."
   (interactive)
   (and (> (string-to-int emacs-version) 18)
        (let ((minibuffer-setup-hook minibuffer-setup-hook)
@@ -2343,7 +2396,11 @@ static char * Gray_X_Face[] = {
 
 ;;;###autoload
 (defun x-face-save (&optional arg)
-  "Save X-Face fields to files."
+  "Extract X-Face headers in the current buffer, encode them to XBM data,
+and save them in the local system.  Directory where XBM files will be
+saved in is specified by the `x-face-image-file-directory-for-save'
+variable.  You can use this command in a summary buffer rather than an
+article buffer for some mail softwares, e.g. Gnus, Wanderlust, etc."
   (interactive "P")
   (let ((x-face-icons-to-color-picture-function
 	 (if arg
@@ -2353,7 +2410,10 @@ static char * Gray_X_Face[] = {
 
 ;;;###autoload
 (defun x-face-view (&optional arg)
-  "View X-Face fields."
+  "Extract X-Face headers in the current buffer, encode them to XBM data,
+and show them using an external image viewer.  You can use this
+command in a summary buffer rather than an article buffer for some
+mail softwares, e.g. Gnus, Wanderlust, etc."
   (interactive "P")
   (let ((x-face-icons-to-color-picture-function
 	 (if arg
@@ -2363,7 +2423,8 @@ static char * Gray_X_Face[] = {
 
 ;;;###autoload
 (defun x-face-ascii-view ()
-  "View X-Face fields as ascii pictures."
+  "Extract X-Face headers in the current buffer, encode them to XBM data,
+and show them as ascii pictures in an Emacs frame."
   (interactive)
   (x-face-save-or-view 'ascii))
 
@@ -2432,7 +2493,7 @@ static char * Gray_X_Face[] = {
 	       (gnus-Subject-show-article))))))
 
 (defun x-face-find-gnus-article-buffer ()
-  "Find article buffer may includes X-Face fields."
+  "Find an article buffer which may includes X-Face headers."
   (or (save-excursion
 	(set-buffer (or (and (boundp 'gnus-article-buffer)
 			     (x-face-buffer-live-p gnus-article-buffer))
@@ -2479,7 +2540,12 @@ static char * Gray_X_Face[] = {
     (error nil))
   (condition-case nil
       (require 'mew-syntax)
-    (error nil)))
+    (error nil))
+  ;; Avoid macro `mew-cache-hit' expand (Mew 1.94.2 or earlier).
+  ;; Changed `mew-cache-hit' from macro to function at Mew 2.0.
+  (if (not (fboundp 'mew-current-get-fld))
+      (setq byte-compile-macro-environment
+	    (cons '(mew-cache-hit . nil) byte-compile-macro-environment))))
 
 (defun x-face-mew-get-article ()
   "Get current message in article buffer."
@@ -2631,12 +2697,16 @@ static char * Gray_X_Face[] = {
 			       (list file))
 		       " "))
 	(sleep-for x-face-viewer-startup-time))
-    (delete-file file)))
+    (delete-file file)
+    (delete-directory (file-name-directory file))))
 
 (defun x-face-sentinel-delete-file (process event)
   (let* ((list (assq process x-face-delete-file-alist))
 	 (file (cdr list)))
-    (and file (file-exists-p file ) (delete-file file))
+    (when (and file
+	       (file-exists-p file))
+      (delete-file file)
+      (delete-directory (file-name-directory file)))
     (setq x-face-delete-file-alist (delq list x-face-delete-file-alist))))
 
 (defun x-face-view-asynchronous-function-for-win95 (file)
@@ -2657,7 +2727,7 @@ static char * Gray_X_Face[] = {
 	      (set-process-sentinel process 'x-face-sentinel-delete-file)))))))
 
 (defun x-face-view-asynchronous-function-for-some (file)
-  "Asynchronous view function for some UN|X."
+  "Asynchronous view function for some UN|X systems."
   (call-process
    "/bin/sh" nil nil nil "-c"
    (mapconcat
@@ -2668,8 +2738,9 @@ static char * Gray_X_Face[] = {
     " "))
   (call-process "/bin/sh" nil nil nil "-c"
 		(format
-		 "(/bin/sleep %d;/bin/rm -f %s)>/dev/null 2>&1 </dev/null&"
-		 x-face-viewer-startup-time file)))
+		 "(/bin/sleep %d;/bin/rm -fr %s)>/dev/null 2>&1 </dev/null&"
+		 x-face-viewer-startup-time
+		 (file-name-directory file))))
 
 
 ;;; Menu.
